/*----------------------------------------------------------------------*/
/* gvtk.h 								*/
/* Copyright (c) 2009  Tim Edwards					*/
/*----------------------------------------------------------------------*/

/*----------------------------------------------------------------------*/
/*      written by Tim Edwards, 6/21/09    				*/
/*----------------------------------------------------------------------*/

#ifndef _GVTK_H
#define _GVTK_H

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>

#ifndef HAVE_U_CHAR
typedef unsigned char	u_char;
typedef unsigned short	u_short;
typedef unsigned int	u_int;
typedef unsigned long	u_long;
#endif

#include "ps.h"

/*----------------------------------------------------------------------*/
/* Deal with 32/64 bit processors based on autoconf results.		*/
/*----------------------------------------------------------------------*/

#if SIZEOF_VOID_P == SIZEOF_UNSIGNED_INT
#define Number(a)		(void *)((u_int) a)
typedef u_int pointertype;
#elif SIZEOF_VOID_P == SIZEOF_UNSIGNED_LONG
#define Number(a)		(void *)((u_long) a)
typedef u_long pointertype;
#else
ERROR: Cannot compile without knowing the size of a pointer.  See gvtk.h.
#endif

/*----------------------------------------------------------------------*/
/* Process forking							*/
/*----------------------------------------------------------------------*/

#ifndef HAVE_VFORK
#define vfork fork
#endif

/*----------------------------------------------------------------------*/
/* Definition shortcuts 						*/
/*----------------------------------------------------------------------*/

#define abs(a)			((a) < 0 ? -(a) : (a))
#define sign(a)			((a) <= 0 ? -1 : 1)
#ifndef min
#define max(a,b)		((a) < (b) ? (b) : (a))
#define min(a,b)		((a) < (b) ? (a) : (b))
#endif

/*----------------------------------------------------------------------*/
/* States of the ghostscript interpreter				*/
/*----------------------------------------------------------------------*/

#define GS_INIT   -1	/* Initial state; gs is idle. */
#define GS_READY   0	/* Drawing done; gs is waiting for "next". */
#define GS_PENDING 1	/* Drawing in progress; gs is busy. */
#define GS_QUEUED  2	/* Queued event needs to be drawn */

/*----------------------------------------------------------------------*/
/* Linked list where we keep information about each parsed file		*/
/*----------------------------------------------------------------------*/

typedef struct _FileList *FileListPtr;

typedef struct _FileList {
   char *filename;
   FILE *fps;
   Boolean is_pdf;
   struct document *doc;
   FileListPtr next;
} FileList;

/*----------------------------------------------------------------------*/
/* The main globally-accessible data structure.  This structure holds	*/
/* all the critical data needed by the drawing window 			*/
/*----------------------------------------------------------------------*/

typedef struct _windowdata *GVWindowDataPtr;

typedef struct _windowdata {

   GVWindowDataPtr next;	/* next window in list */

   /* widgets and X11 parameters */
   Tk_Window	area;
   Window	window;
   GC		gc;

   short	width, height;		/* Width/height of window */

   FileListPtr	fileptr;	/* Pointer to structure for file */
				/* being displayed in the window */
   int		page;		/* Page of file being shown (starting 0) */

   float	zoom;		/* Zoom factor */
   int		rotate;		/* Rotation (0, 90, 180, etc.) */
   int		offsetx;	/* Translation (in PS units) */	
   int		offsety;

   Tcl_Channel	gs_channel;	/* Pipe to window's ghostscript interpreter */
   int		gs_state;	/* State of the ghostscript interpreter */
   Window	gs_mwin;	/* Window # used by ghostscript for messages */

   short        pixwidth, pixheight;	/* Width/height of pixmap */
   Pixmap	dbuf;			/* Pixmap drawable */

} GVWindowData;

/*----------------------------------------------------------------------*/
/* A convenient structure for holding the window and file lists		*/
/*----------------------------------------------------------------------*/

typedef struct {
   GVWindowData *windowlist;		/* Linked list of known windows */
   FileListPtr  filelist;		/* List of PostScript files	*/
   Tk_Window	topwin;			/* Top-level window of the app  */
} Globaldata;

/*----------------------------------------------------------------------*/
/* External variable declarations					*/
/*----------------------------------------------------------------------*/

extern Globaldata gvobjs;
extern GVWindowData *areawin;
extern Tcl_Interp *gvinterp;
extern Display *dpy;
extern Tcl_HashTable GvTagTable;

/*----------------------------------------------------------------------*/
/* Function prototypes							*/
/*----------------------------------------------------------------------*/

extern void start_gs(GVWindowData *);
extern int exit_gs(GVWindowData *, Boolean);
extern void ghostinit_local(GVWindowData *);
extern int renderpage(GVWindowData *);
extern int copypage(GVWindowData *);
extern int copyoffset(GVWindowData *, int, int);

GVWindowData *new_ghostscript_target(Tcl_Interp *, Tk_Window);
FileList *new_file(char *);

extern int Tk_SimpleObjCmd();
extern int GvTagCallback();
extern int GvInternalTagCall(Tcl_Interp *, int, ...);

extern int gvtk_config();
extern int gvtk_start();
extern int gvtk_render();
extern int gvtk_quit();
extern int gvtk_page();
extern int gvtk_document();
extern int gvtk_zoom();
extern int gvtk_rotate();
extern int gvtk_offset();
extern int gvtk_tag();

/*----------------------------------------------------------------------*/

#endif /* _GVTK_H */
