/*----------------------------------------------------------------------*/
/* gvtkexec.c								*/
/*									*/
/* Written by R. Timothy Edwards, June 2009.				*/
/* This file mainly lifted from the main application routine for	*/
/* "wish" from the Tk distribution.					*/
/*									*/
/* This is a compact re-write of the "wish" executable that calls	*/
/* Tk_MainEx with application-specific processing.  Specifically, 	*/
/* "wish" doesn't allow the startup script (~/.wishrc) to be renamed.	*/
/* However, for gvtk running as an extension of Tcl, we want to	*/
/* source the gvtk.tcl file instead of ~/.wishrc.  So, all this	*/
/* file really does is to set the Tcl variable "tcl_rcFileName" to	*/
/* gvtk.tcl, so that it will be processed as the startup script,	*/
/* followed by a drop back to the Tcl interpreter command-line main	*/
/* loop.								*/
/*----------------------------------------------------------------------*/

#include <stdio.h>

#include <tk.h>
#include <tcl.h>

/*----------------------------------------------------------------------*/
/* Application initiation.  This is exactly like the AppInit routine	*/
/* for "wish", minus the cruft, but with "tcl_rcFileName" set to	*/
/* "gvtk.tcl" instead of "~/.wishrc".				*/
/*----------------------------------------------------------------------*/

int
gvtk_AppInit(interp)
    Tcl_Interp *interp;
{
    if (Tcl_Init(interp) == TCL_ERROR) {
	return TCL_ERROR;
    }
    if (Tk_Init(interp) == TCL_ERROR) {
	return TCL_ERROR;
    }
    Tcl_StaticPackage(interp, "Tk", Tk_Init, Tk_SafeInit);

    /* This is where we replace the home ".wishrc" file with	*/
    /* gvtk's startup script.					*/

    Tcl_SetVar(interp, "tcl_rcFileName", SCRIPTS_DIR "/gvtk.tcl",
		TCL_GLOBAL_ONLY);
    return TCL_OK;
}

/*----------------------------------------------------------------------*/
/* The main procedure;  replacement for "wish".				*/
/*----------------------------------------------------------------------*/

int
main(argc, argv)
   int argc;
   char **argv;
{
    Tk_Main(argc, argv, gvtk_AppInit);
    return 0;
}

/*----------------------------------------------------------------------*/
